// PG_EntryListInfo.cpp: implementation of the 
// PG_EntryListConfig and PG_AttrTSpec classes.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "PCPages32.h"
#include "PG_EntryListInfo.h"
#include "QDN.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// PG_ColAttrTSpec Construction/Destruction
//////////////////////////////////////////////////////////////////////

PG_ColAttrTSpec::PG_ColAttrTSpec( QAttrT *pAttrT, BOOL bPrefRDN )
{
	m_pAttrT = pAttrT;
	m_bPrefRDN = bPrefRDN;
}

PG_ColAttrTSpec::~PG_ColAttrTSpec()
{
}

BOOL PG_ColAttrTSpec::GetEntryValue( QEntry *pEntry, CString &rStrVal )
{
	//
	// Check for specification strings. Only 'RDN' supported at present.
	if ( m_bPrefRDN == TRUE )
		//
		// If the RDN attribute is preferred then try finding the attribute
		// type in the RDN. If not found then just return the friendly RDN
		// value.
		{
			//
			// Get the RDN
			QRDN rdn;
			pEntry->DN().GetRDN( rdn );

			//
			// Get the RDN correct attribute 
			QRDNAttr *pRDNAttr;
			POSITION pos = rdn.GetHeadRDNAttr();
			while ( pos != NULL )
				{
					pRDNAttr = rdn.GetNextRDNAttr( pos );
					if ( pRDNAttr->Type() == m_pAttrT )
						break;
				}

			if ( pos != NULL )
				{
					pRDNAttr->Value()->Print( rStrVal );
					return TRUE;
				}

			//
			// Return the RDN entire
			rdn.GetFriendlyRDNString( rStrVal );
		}

	//
	// Default action
	POSITION pos = pEntry->GetHeadPosition();
	while ( pos != NULL )
		{
			QAttr *pAttr = pEntry->GetNextAttr( pos );
			if ( pAttr->Type() == m_pAttrT )
				{
					QStringList valStrList;
					pAttr->PrintValues( valStrList );

					if ( valStrList.IsEmpty() )
						return FALSE;

					rStrVal = valStrList.GetHead();
					return TRUE;
				}
		}

	return FALSE;
}

//////////////////////////////////////////////////////////////////////
// PG_ColumnSpec Construction/Destruction
//////////////////////////////////////////////////////////////////////

PG_ColumnSpec::PG_ColumnSpec()
{
	m_pOC = NULL;
}

PG_ColumnSpec::~PG_ColumnSpec()
{
	POSITION pos = m_CATSpecList.GetHeadPosition();
	while ( pos != NULL )
		delete m_CATSpecList.GetNext( pos );
}

//////////////////////////////////////////////////////////////////////
// Set the column specification. Fails if invalid attribute types
// supplied in the column specification list.
BOOL PG_ColumnSpec::SetColumnSpec( QOC *pOC, UINT nIconID, QStringList &rSpecList )
{
	extern CPCPages32App theApp;

	//
	// Set the object class
	m_pOC = pOC;

	//
	// Read each column cell specification
	POSITION pos = rSpecList.GetHeadPosition();
	while ( pos != NULL )
		{
			//
			// Split the column specification into a list (of at most two) words.
			CString colSpecStr = rSpecList.GetNext( pos );
			QStringList colSpecWords;
			colSpecWords.SetSplitString( colSpecStr, ':' );

			//
			// The first element in the list should be an attribute type string
			POSITION wordPos = colSpecWords.GetHeadPosition();
			QAttrT *pAttrT = theApp.DS()->Table()->str2at( colSpecWords.GetNext( pos ) );

			//
			// The second is an optional specification string (only 'RDN' supported at present)
			CString specStr = "";
			if ( pos != NULL )
				specStr = colSpecWords.GetNext( pos );

			//
			// Add the new column
			/// XXX PG_ColAttrTSpec *pColAttrTSpec = new PG_ColAttrTSpec( pAttrT, specStr );
			/// XXX m_CATSpecList.AddTail( pColAttrTSpec );
		}

	return TRUE;
}

BOOL PG_ColumnSpec::GetValue( QEntry *pEntry, UINT nColID, CString &rStrVal )
{
	POSITION pos = m_CATSpecList.FindIndex( nColID );
	if ( pos == NULL )
		return FALSE;

	PG_ColAttrTSpec *pCATSpec = m_CATSpecList.GetAt( pos );
	return pCATSpec->GetEntryValue( pEntry, rStrVal );
}

//////////////////////////////////////////////////////////////////////
// PG_EntryListConfig Construction/Destruction
//////////////////////////////////////////////////////////////////////

PG_EntryListConfig::PG_EntryListConfig()
{
}

PG_EntryListConfig::~PG_EntryListConfig()
{
	POSITION pos = m_colATList.GetHeadPosition();
	while ( pos != NULL )
		{
			delete m_colATList.GetNext( pos );
		}
}

//////////////////////////////////////////////////////////////////////
// PG_EntryListConfig public interface
//////////////////////////////////////////////////////////////////////

BOOL PG_EntryListConfig::ReadConfiguration()
{
	extern CPCPages32App theApp;
	HKEY hColKey = theApp.OpenAppKey( "Entry List Columns" );
	if ( hColKey == NULL )
		{
#ifdef _DEBUG
			TRACE0
				( "Failed to read key:\nHKCU/Software/Brunel University/PC-Pages/Entry List Columns\n" );
#endif
			RegCloseKey( hColKey );
			return FALSE;
		}

	//
	// Get the column list
#define KEYVAL_BUFLEN 4096
	DWORD dwValueType;
	BYTE dataBuf[KEYVAL_BUFLEN];
	DWORD dwDataBufLen = KEYVAL_BUFLEN;
	LONG ok = RegQueryValueEx
		(
			hColKey,            // handle to key to query
			"Column List",  // address of name of value to query
			NULL,   // reserved
			&dwValueType, // address of buffer for value type
			dataBuf,      // address of data buffer
			&dwDataBufLen		// address of data buffer size
		);

	if ( ok != ERROR_SUCCESS )
		{
#ifdef _DEBUG
			TRACE0
				( "Failed to read registry key:\nHKCU/Software/Brunel University/PC-Pages/Entry List Columns/Column List\n" );
#endif
			RegCloseKey( hColKey );
			return FALSE;
		}

	//
	// Parse and store the column names
	m_colNameList.SetSplitString( CString( reinterpret_cast<char *>( dataBuf ) ), ',' );

	//
	// Read the attribute type associated with each column
	POSITION pos = m_colNameList.GetHeadPosition();
	while ( pos != NULL )
		{
			DWORD dwDataBufLen = KEYVAL_BUFLEN;
			CString colName = m_colNameList.GetNext( pos );
			colName.TrimLeft();
			colName.TrimRight();
			ok = RegQueryValueEx
					(
						hColKey,            // handle to key to query
						colName,  // address of name of value to query
						NULL,   // reserved
						&dwValueType, // address of buffer for value type
						dataBuf,      // address of data buffer
						&dwDataBufLen		// address of data buffer size
					);

			if ( ok != ERROR_SUCCESS )
				{
#ifdef _DEBUG
					TRACE
						( "Failed to read registry value:\nHKCU/Software/Brunel University/PC-Pages/Entry List Columns/%s\n",
							colName );
#endif
					RegCloseKey( hColKey );
					return FALSE;
				}

			//
			// Get the column specification string, which is an attribute type name
			// followed by optional indicators.
			CString colSpecStr( reinterpret_cast<char *>( dataBuf ) );
			QStringList specList;

			specList.SetSplitString( colSpecStr, ':' );
			if ( specList.GetCount() == 0 )
				//
				// Duff column specification
				{
#ifdef _DEBUG
					TRACE
						( "Bad column specification found in registry value:\
							 \nHKCU/Software/Brunel University/PC-Pages/Entry List Columns/%s\n",
							colName );
#endif
					
					return FALSE;
				}

			//
			// Get the attribute type
			CString colATStr = specList.GetHead();
			QAttrT *pAttrT = theApp.DS()->Table()->str2at( colATStr );
			if ( pAttrT == NULL )
				{
#ifdef _DEBUG
					TRACE
						( "Unrecognised attribute type found in registry value:\
							 \nHKCU/Software/Brunel University/PC-Pages/Entry List Columns/%s\n",
							colName );
#endif
					return FALSE;
				}

			//
			// Get indicators
			BOOL bPrefRDN = FALSE;
			POSITION specPos = specList.FindIndex( 1 );
			if ( specPos != NULL )
				{
					CString prefStr = specList.GetAt( specPos );
					if ( prefStr == "RDN" )
						bPrefRDN = TRUE;
				}

			//
			// Create the column specification
			PG_ColAttrTSpec *pCAT = new PG_ColAttrTSpec( pAttrT, bPrefRDN );
			m_colATList.AddTail( pCAT );
		}

	return TRUE;
}

//////////////////////////////////////////////////////////////////////
// PG_EntryListConfig::GetColumnValue
// Get the string value that plugs into the identified column
// for the given entry. 
BOOL PG_EntryListConfig::GetColumnValue( QEntry *pEntry, UINT nColID, CString &rStrVal )
{
	rStrVal = "";

	POSITION pos = m_colATList.FindIndex( nColID );
	if ( pos == NULL )
		return FALSE;

	PG_ColAttrTSpec *pCAT = m_colATList.GetAt( pos );
	pCAT->GetEntryValue( pEntry, rStrVal );

	return TRUE;
}

//////////////////////////////////////////////////////////////////////
// PG_EntryListConfig::GetIconID
BOOL PG_EntryListConfig::GetIconID( QEntry *pEntry, UINT &nIconID )
{
	// XXX PG_EntryListConfig::GetIconID unimplemented
	return FALSE;
}

//////////////////////////////////////////////////////////////////////
// PG_EntryListConfig::GetEntryValue
// Retrieve just one value of a given attribute type from an entry
BOOL PG_EntryListConfig::GetEntryValue( QEntry *pEntry, QAttrT *pAttrT, CString &rStrVal )
{
	rStrVal = "";

	POSITION pos = pEntry->GetHeadPosition();
	while ( pos != NULL )
		{
			QAttr *pAttr = pEntry->GetNextAttr( pos );
			if ( pAttr->Type() == pAttrT )
				{
					QStringList valStrList;
					pAttr->PrintValues( valStrList );

					if ( valStrList.IsEmpty() )
						return FALSE;

					rStrVal = valStrList.GetHead();
					return TRUE;
				}
		}

	return FALSE;
}

