// PG_FlashWnd.cpp : implementation file
//

#include "stdafx.h"
#include "PCPages32.h"
#include "PG_FlashWnd.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// PG_FlashWnd
PG_FlashWnd::PG_FlashWnd()
{
	m_nStep = -1;
}

PG_FlashWnd::~PG_FlashWnd()
{
}

BEGIN_MESSAGE_MAP(PG_FlashWnd, CWnd)
	//{{AFX_MSG_MAP(PG_FlashWnd)
	ON_WM_CREATE()
	ON_WM_PAINT()
	ON_WM_TIMER()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// PG_FlashWnd message handlers
BOOL PG_FlashWnd::Create(LPCTSTR lpszClassName, LPCTSTR lpszWindowName, DWORD dwStyle, const RECT& rect, CWnd* pParentWnd, UINT nID, CCreateContext* pContext) 
{
	return CWnd::Create
		(lpszClassName, lpszWindowName, dwStyle, rect, pParentWnd, nID, pContext);
}

/////////////////////////////////////////////////////////////////////////////
// PG_FlashWnd::OnCreate
// Make sure the window has the transparent window style.
int PG_FlashWnd::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CWnd::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	//
	// Add transparent style
	ModifyStyleEx( 0, WS_EX_TRANSPARENT );
	return 0;
}

/////////////////////////////////////////////////////////////////////////////
// PG_FlashWnd::Flash
// Create the transparent window used to draw the zoom effect. Pre-alculate
// a few things and create the timer used to draw the zoom steps.
void PG_FlashWnd::Flash( CWnd *pParentWnd, CRect &rFromRect, CRect &rToRect )
{
	RECT rect;

	pParentWnd->GetClientRect( &rect );
	Create( NULL, "Flash", WS_CHILD|WS_VISIBLE, rect, pParentWnd, 9999, NULL );

	m_fromRect = rFromRect;
	m_toRect = rToRect;

	m_fLStep = static_cast<float>( rToRect.left - rFromRect.left );
	m_fRStep = static_cast<float>( rToRect.right - rFromRect.right );
	m_fTStep = static_cast<float>( rToRect.top - rFromRect.top );
	m_fBStep = static_cast<float>( rToRect.bottom - rFromRect.bottom );

	m_stepNum = 20;
	m_fLStep /= m_stepNum;
	m_fRStep /= m_stepNum;
	m_fTStep /= m_stepNum;
	m_fBStep /= m_stepNum;

	//
	// Create the timer
	m_nStep = 0;
	SetTimer( 1, 10, NULL );
}

/////////////////////////////////////////////////////////////////////////////
// PG_FlashWnd::OnPaint
// Handle paint messages.
void PG_FlashWnd::OnPaint() 
{
	CPaintDC dc( this ); // device context for painting
	DrawZoomFrame( dc );
}
	
/////////////////////////////////////////////////////////////////////////////
// PG_FlashWnd::DrawZoomFrame
// Draw the current zoom frame.
void PG_FlashWnd::DrawZoomFrame( CDC &rDC )
{
	if ( m_nStep == -1 )
		return;

	CRect rect;
	rect.left = m_fromRect.left + static_cast<int>( m_fLStep * m_nStep );
	rect.right = m_fromRect.right + static_cast<int>( m_fRStep * m_nStep );
	rect.top = m_fromRect.top + static_cast<int>( m_fTStep * m_nStep );
	rect.bottom = m_fromRect.bottom + static_cast<int>( m_fBStep * m_nStep );

	CSize sz;
	sz.cx = sz.cy = 2;

	RECT *pLastRect;
	pLastRect = ( m_nStep == 0 ? NULL : m_lastRect );

	rDC.DrawDragRect( rect, sz, pLastRect, sz );
	m_lastRect = rect;
}

void PG_FlashWnd::OnTimer(UINT nIDEvent) 
{
	RedrawWindow( NULL, NULL, RDW_INVALIDATE | RDW_UPDATENOW | RDW_ERASE ); 

	CClientDC dc( this );
	DrawZoomFrame( dc );

	if ( ++m_nStep == m_stepNum )
		{
			KillTimer( 1 );
			DestroyWindow();
			m_nStep = -1;
		}
	else
		CWnd::OnTimer(nIDEvent);
}
