// QBindManager.h: interface for the QBindManager class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_QBINDMANAGER_H__31AFB0A3_A6FD_11D2_89E9_980EE8619202__INCLUDED_)
#define AFX_QBINDMANAGER_H__31AFB0A3_A6FD_11D2_89E9_980EE8619202__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "QConnection.h"
#include "QError.h"
#include "ptrlist.h"

class QConnMap
{
public:
	QConnMap();
	virtual ~QConnMap();

public:
  BOOL Lookup( ConnID id, QConnection *&conn );
  void SetAt( ConnID id, QConnection *conn );
  POSITION GetHeadPosition()
		{
			WIN32_GUFF();
			return conn_list.GetHeadPosition() ;
		} ;

  QConnection *GetNext( POSITION &pos)
		{
			WIN32_GUFF();
			return conn_list.GetNext( pos );
		} ;

private:
  PtrList<QConnection> conn_list;
};

class QBindManager
{
public:
	QBindManager();
	virtual ~QBindManager();

public:
	// Specify a connection
	ConnID SpecifyConnection
		( CString &host, int port, CString &user, CString &password );

	// Housekeeping
	void TidyUp();

  // Async bind interface
  BOOL BindAsync( ConnID conn_id, QError *&err );
	BOOL RebindAsync( ConnID conn_id, QError *&err );
  BOOL BindWait( ConnID id, QError *&err );
	ConnID NotifySocketClose( SOCKET socket );

public:
  // Connection status
  BOOL Bound( ConnID id );

  // Compare a recorded connection's details with the given ones
  BOOL Exists( ConnID &id, const CString &host, int port, const CString &user, const CString &passwd );

  // Get LDAP context. Result shouldn't be stored as the pointer will change
  // as the connection is closed (due to inactivity) and then re-opened
  LDAP *GetLDAPContext( ConnID id );

	// Get the connection block
	QConnMap *GetConnectionMap() { return &conn_map ; } ;

	// Get the details of the identified connection
	BOOL GetConnection
		( ConnID conn_id, CString &host, int &port, CString &user, CString &password );

public:
	// Connection reference counting
	void AddRef( ConnID conn_id );
	void LoseRef( ConnID conn_id );

public:
  // Unbind operations
  void UnbindAll();
	void Unbind( ConnID conn_id );

public:
	// Dump a connection to the log file
	void Dump( ConnID conn_id );

protected:
  // Mapping from ConnIDs to QConnections
  QConnMap conn_map;

private:
  // Connection ID accumulator
  ConnID conn_acc_id;

	// Idle timeout value
	int m_nIdleTimeout;
};

#endif // !defined(AFX_QBINDMANAGER_H__31AFB0A3_A6FD_11D2_89E9_980EE8619202__INCLUDED_)
