// QDSManager.h: interface for the QDSManager class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_DSMANAGER_H__35680AA1_AC14_11D2_89E9_D1B8AB62BD1A__INCLUDED_)
#define AFX_DSMANAGER_H__35680AA1_AC14_11D2_89E9_D1B8AB62BD1A__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "Oidtable.h"
#include "QBindManager.h"
#include "QRequest.h"
#include "QFormConfiguration.h"
#include "QLogFile.h"

class AFX_EXT_CLASS QDSManager  
{
public:
	QDSManager();
	~QDSManager();

public:
	BOOL PreInitialise();
  BOOL Initialise( CString &dn, CString &host, int port );

public:
  void Poll();
	void TidyUp();
	BOOL Waiting();
	BOOL WaitingOn( ConnID conn_id );
	void NotifySocketClose( SOCKET socket );
	void NotifyConnectionClose( ConnID conn_id );

public:
	// Request registration
  void RegisterRequest( QRequest *request );
  void DeregisterRequest( QRequestID req_id );

public:
	// Abort procedures
  void AbortRequest( QRequestID req_id );
  void AbortAll();

public:
	// General request status 
  QRequest *GetRequest( QRequestID req_id );
	QRequestID GenRequestID();
	QRequest *GetOpOwner( LDAP *ld, int ldap_msgid );

public:
	// Member access
  Oidtable *Table() { return &m_oidtable; } ;
  QBindManager *BindManager() { return &m_bindMgr; } ;
	QFormConfiguration *FormConfig() { return m_pFormConfig; } ;

public:
	// Logging
	void Open( const char *pFileName );
	void Log( const char *pFormatStr, ... );

private:
  // Manager for open server connections
  QBindManager m_bindMgr;

  // The oidtable
  Oidtable m_oidtable;

  // Form search configuration
  QFormConfiguration *m_pFormConfig;

  // State flags
  BOOL m_bPolling;

  // Accumulator used to generate new request identifiers
  QRequestID m_nReqIDAcc;

  // Request list
  PtrList<QRequest> m_requestList;
};

#endif // !defined(AFX_DSMANAGER_H__35680AA1_AC14_11D2_89E9_D1B8AB62BD1A__INCLUDED_)
