// QFormConfiguration.h: interface for the QFormSearch class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_QFORMCONFIGURATION_H__INCLUDED_)
#define AFX_QFORMCONFIGURATION_H__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "stdafx.h"
#include "QSearchClass.h"
#include "QFormField.h"
#include "QRuleSet.h"

// Forward references
class QDSManager;

// The QForm class describes a single type of user search. This includes
// the following pieces of information:
//
// o A form description.
// o A search Heuristics.
class AFX_EXT_CLASS QForm
{
public:
  QForm
    ( const CString &name, QFormInput *target, PtrList<QFormInput> &context, QRuleSet *rule_set );

  ~QForm();

public:
  const CString &Name() { return name ; } ;
  QRuleSet *RuleSet() { return rule_set ; } ;
  QFormInput *Target() { return target ; } ;
  void Context( PtrList<QFormInput> &context );
	void BuildInputFields( QFormInput **target, PtrList<QFormInput> &context );

public:
	CString &TipString() { return tip_str; } ;
	CString &HelpString() { return help_str; } ;

private:
  CString name;
  QFormInput *target;
  PtrList<QFormInput> context;
  QRuleSet *rule_set;
	CString tip_str;
	CString help_str;
};

// The class holding all configuration information on form based searching.
// The Load() method loads from configuration store (be it disk, directory, registry, etc.)
class AFX_EXT_CLASS QFormConfiguration
{
public:
  QFormConfiguration( QDSManager *ds_arg );
  virtual ~QFormConfiguration();

public:
  void GetFormList( CStringList &form_name_list );
  QForm *GetForm( CString &form_name );

public:
  BOOL LoadConfiguration( CString &config_dn, CString &host, int port );

private:
  BOOL AddForm
		( CString &name,
			QFormInput *target,
			PtrList<QFormInput> &context,
			CString &rule_set_name,
			CString &tip_str,
			CString &help_str );

  BOOL AddRuleSet( CString &name, PtrList<QSearchClass> &class_list );
  QRuleSet *GetRuleSet( CString &name );

private:
	BOOL LoadForms( LDAP *ld, CString base_dn );

	QFormInput *QFormConfiguration::LoadFormInput
		( LDAP *ld, char *form_dn_str, CString &input_name, QRuleSet *rule_set );

	QFormField *QFormConfiguration::LoadFormField
		( LDAP *ld, char *input_dn_str, CString &field_name );

	BOOL LoadRuleSets( LDAP *ld, CString base_dn );
	BOOL LoadSearchClasses( LDAP *ld, CString base_dn, PtrList<QSearchClass> &sc );

private:
  QDSManager *ds;
  PtrList<QForm> form_list;
  PtrList<QRuleSet> rule_set_list;
};

#endif // AFX_QFORMCONFIGURATION_H__INCLUDED_
