// QFormField.cpp: interface implementation for the QFormField 
// and QFormInput classes.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "QFormField.h"
#include "QSearchClass.h"

////////////////////////////////////////////////////////////////////////////////
// Construction/destruction - QFormField
////////////////////////////////////////////////////////////////////////////////

QFormField::QFormField
	( CString &field_name_arg, CString &default_value_arg, PtrList<QAttrT> &filt_attrt_list_arg )
{
  WIN32_GUFF();

	field_name = field_name_arg;
	input_value = default_value_arg;

	POSITION pos = filt_attrt_list_arg.GetHeadPosition();
	while ( pos != NULL )
		{
			filt_attrt_list.AddTail( filt_attrt_list_arg.GetNext( pos ) );
		}
}

QFormField::~QFormField()
{
  WIN32_GUFF();

  filt_attrt_list.RemoveAll();
}

////////////////////////////////////////////////////////////////////////////////
// Create a copy of this field. Used by QFormInput::CopySelf.
QFormField *QFormField::CopySelf()
{
  return new QFormField( field_name, input_value, filt_attrt_list );
}

////////////////////////////////////////////////////////////////////////////////
// Generate a filter string from a search string and the attribute types
// in the "filter on" list (filt_attr_list).
BOOL QFormField::GenerateFilterString( CString &filt_str )
{
  WIN32_GUFF();

	CString str = Value();
	QStringList word_list;

	//
	// Split search string into words
	str.TrimLeft();
	str.TrimRight();

	if ( str.IsEmpty() || str == "" )
		return FALSE;

	word_list.SetSplitString( str, ' ' );

	//
	// Create a filter string for each attribute type in the 
	// "filter on" list (filt_attr_list)
	filt_str = "";

	POSITION pos = filt_attrt_list.GetHeadPosition();
	while ( pos != NULL )
		{
			QAttrT *at = filt_attrt_list.GetNext( pos );
			CString tmp_str = "(" + at->Name() + "=" ;

			POSITION word_pos = word_list.GetHeadPosition();
			while ( word_pos != NULL )
				{
					CString word = word_list.GetNext( word_pos );
					tmp_str += ( "*" + word );
				}

			filt_str = filt_str + tmp_str;
			filt_str = filt_str + "*)";
		}

	if ( filt_attrt_list.GetCount() > 1 )
		filt_str = ( "(&" + filt_str + ")" );

	return TRUE;
}

////////////////////////////////////////////////////////////////////////////////
// Construction/destruction - QFormInput
////////////////////////////////////////////////////////////////////////////////

QFormInput::QFormInput
	( CString &input_name_arg, QSearchClass *srchclass_arg, PtrList<QFormField> &field_list_arg )
{
  WIN32_GUFF();

	input_name = input_name_arg;
	srchclass = srchclass_arg;

	POSITION pos = field_list_arg.GetHeadPosition();
	while ( pos != NULL )
  	field_list.AddTail( field_list_arg.GetNext( pos ) );
}

QFormInput::~QFormInput()
{
  WIN32_GUFF();

	POSITION pos = field_list.GetHeadPosition();
	while ( pos != NULL )
  	delete field_list.GetNext( pos );
}

////////////////////////////////////////////////////////////////////////////////
// Create a copy of the class. This is used to build a copy of the form that
// the Query engine client can use to "fill in" and then send to
// QFormSearch as input.
QFormInput *QFormInput::CopySelf()
{
  WIN32_GUFF();

  PtrList<QFormField> field_list_copy;

  POSITION pos = field_list.GetHeadPosition();
  while ( pos != NULL )
    {
      field_list_copy.AddTail( field_list.GetNext( pos )->CopySelf() ); 
    }

  return new QFormInput( input_name, srchclass, field_list_copy );
}

////////////////////////////////////////////////////////////////////////////////
// Get the list of fields contained in this input group.
UINT QFormInput::GetFieldNames( QStringList &field_name_list )
{
  WIN32_GUFF();

	field_name_list.RemoveAll();
  UINT field_count = 0;

	POSITION pos = field_list.GetHeadPosition();
	while ( pos != NULL )
		{
			QFormField *field = field_list.GetNext( pos );
			field_name_list.AddTail( field->Name() );
      field_count++;
		}

  return field_count;
}

////////////////////////////////////////////////////////////////////////////////
// Return a *reference* to the string valued contained in the named field.
BOOL QFormInput::SetFieldValue( CString &field_name, CString &val )
{
  WIN32_GUFF();

	POSITION pos = field_list.GetHeadPosition();
	while ( pos != NULL )
		{
			QFormField *field = field_list.GetNext( pos );
			if ( field->Name() == field_name )
				{
					field->Value() = val;
          return TRUE;
				}
		}

  return FALSE;
}

////////////////////////////////////////////////////////////////////////////////
// Generate the combined filter covering all fields within this input group.
BOOL QFormInput::GenerateFilterString( CString &filt_str )
{
  WIN32_GUFF();

  filt_str.Empty();
  int filt_count = 0;
  POSITION pos = field_list.GetHeadPosition();
  while ( pos != NULL )
    {
      CString tmp_filt;
      QFormField *field = field_list.GetNext( pos );
      if ( field->GenerateFilterString( tmp_filt ) == TRUE )
        {
          switch ( filt_count++ )
            {
            case 0:
              //
              // First filter component can stand alone
              filt_str = tmp_filt;
              break;

            case 1:
              //
              // Second filter item means that we have to wrap the
              // filter with AND logic
              filt_str = "(&" + filt_str + tmp_filt;
              break;

            default:
              //
              // Subsequent filter components can just be tacked
              // onto the end
              filt_str += tmp_filt;
              break;
            }
        }
    }

  //
  // Did everything go OK?
  if ( filt_count == 0 )
		{
#ifdef _DEBUG
			CString debug_msg;
			debug_msg.Format( "Failed to generate filter for form input '%s'\n", Name() );
			OutputDebugString( debug_msg );
#endif
			return FALSE;
		}

  // 
  // Add a closing parenthesis if this is a multi-component filter
  if ( filt_count > 1 )
    filt_str += ")";

  //
  // Prepend the object class filter component from the search class
  CString oc_filt;
  if ( srchclass->GenerateFilterString( oc_filt ) == FALSE )
    {
#ifdef _DEBUG
      CString msg;
      msg.Format( "Failed to generate filter for searchclass '%s'\n", srchclass->Name() );
      OutputDebugString( msg );
#endif
      return FALSE;
    }

  filt_str = "(&" + oc_filt + filt_str + ")";

#ifdef _DEBUG
  CString debug_msg;
  debug_msg.Format( "Generate filter:\n%s\n", filt_str );
  OutputDebugString( debug_msg );
#endif

  return TRUE;
}

