// QFormSearch.cpp: implementation of the QFormSearch class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Query32.h"
#include "QFormSearch.h"
#include "QFormCompletionReport.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction
QFormSearch::QFormSearch
  ( QDSManager *ds_arg, QRequestID req_id_arg, QRequester *requester_arg,
		QForm *form_arg, QFormInput *ffi_target_arg, PtrList<QFormInput> &ffi_list_arg )
: 
QRequest( ds_arg, req_id_arg, requester_arg )
{
  WIN32_GUFF();

  form = form_arg;

  // Input 
  ffi_target = ffi_target_arg;
  POSITION pos = ffi_list_arg.GetHeadPosition();
  while ( pos != NULL )
    ffi_list.AddTail( ffi_list_arg.GetNext( pos ) );
}

//////////////////////////////////////////////////////////////////////
// Destruction
QFormSearch::~QFormSearch()
{
  WIN32_GUFF();

	//
	// Abort any outstanding operations
	if ( Finished() == FALSE )
		Abort();

	//
	// Delete operation records
	POSITION pos = srch_node_list.GetHeadPosition();
	while ( pos != NULL )
		{
			QFormSearchNode *node = srch_node_list.GetNext( pos );
			delete node;
		}

	srch_node_list.RemoveAll();

	//
  // Delete input
  pos = ffi_list.GetHeadPosition();
  while ( pos != NULL )
    {
      QFormInput *ffi = ffi_list.GetNext( pos );
      delete ffi;
    }

  if ( ffi_target != NULL )
    delete ffi_target;
}

//////////////////////////////////////////////////////////////////////
// Invoke a form search with the given input
BOOL QFormSearch::WaitingOn( ConnID conn_id )
{
	WIN32_GUFF();

	POSITION pos = srch_node_list.GetHeadPosition();
	while ( pos != NULL )
		{
			QFormSearchNode *pNode = srch_node_list.GetNext( pos );
			if ( pNode->Invoked() == TRUE && 
						pNode->Done() == FALSE &&
						pNode->GetConnID() == conn_id )
				return TRUE;
		}

	return FALSE;
}

//////////////////////////////////////////////////////////////////////
// Invoke a form search with the given input
BOOL QFormSearch::Invoke( QError *& err )
{
  WIN32_GUFF();

  //
  // Record invocation and set initial state
  QRequest::invoked = TRUE;
	m_nOpsInvoked = 0;
	m_nOpsFailed = 0;

  //
  // Find *specific* search classes that match any of the given input
  PtrList<QSearchClass> sc_list, base_list;
  QRuleSet *rule_set = form->RuleSet();
  rule_set->GetClassList( sc_list );

  POSITION pos = sc_list.GetHeadPosition();
  while ( pos != NULL )
    {
      QSearchClass *sc = sc_list.GetNext( pos );
			QDN dn;
      if ( sc->IsSpecific( dn ) == TRUE )
        base_list.AddTail( sc );
    }
  
  if ( base_list.GetCount() == 0 )
    // Can't continue if no search base points are provided!
    {
#ifdef _DEBUG
      CString msg;
      msg.Format
        ( "Form search invocation failed : no base points found for form '%s'.\n", form->Name() );

			OutputDebugString( msg );
#endif
      err = new QError( QERR_INSUFFICIENT_DATA );
      return FALSE;
    }

	//
  // Go through form field inputs to see which we can use to form a search based
  // at any of the found base points. These are stuffed into usable_ffi_list.
	//
	// First create a list of ALL inputs, i.e. target input
	// plus context inputs...
  PtrList<QFormInput> usable_ffi_list;
	PtrList<QFormInput> full_ffi_list;

	full_ffi_list.AddTail( ffi_target );

  POSITION ffi_pos = ffi_list.GetHeadPosition();
	while ( ffi_pos != NULL )
		full_ffi_list.AddTail( ffi_list.GetNext( ffi_pos ) );

  UINT node_count = 0;
  UINT errs = 0;

	ffi_pos = full_ffi_list.GetHeadPosition();
  while ( ffi_pos != NULL )
    {
      QFormInput *ffi = full_ffi_list.GetNext( ffi_pos );

      POSITION base_pos = base_list.GetHeadPosition();
      while ( base_pos != NULL )
        {
          QSearchClass *sc = base_list.GetNext( base_pos );

          //
          // Can this base point be used to initiate a search for this field?
          if ( sc->CanSearchFor( ffi->SearchClass() ) )
            {
              //
              // Generate a list of remaining form field inputs. This is the full form
              // field input list (this->ffi_list) less the current 'usable' form field
              // input (ffi).
              PtrList<QFormInput> ffi_left_list;
							QFormInput *ffi_target_left =
									( ffi == ffi_target? NULL : ffi_target );

              POSITION ffi_pos2 = ffi_list.GetHeadPosition();
              while ( ffi_pos2 != NULL )
                {
                  QFormInput *left_ffi = ffi_list.GetNext( ffi_pos2 );
                  if ( left_ffi != ffi )
                    ffi_left_list.AddTail( left_ffi );
                }

              //
              // Generate search filter from search class and form field input string
              CString filter_str;
              if ( ffi->GenerateFilterString( filter_str ) == FALSE )
                {
#ifdef _DEBUG
                  CString msg;
                  msg.Format
                    ( "Search filter generation failed : search class '%s'\n",
											sc->Name() );

									OutputDebugString( msg );
#endif

                  // Can't specify a search operation without a filter...
                  errs++;
                  continue;
                }

              //
              // Initiate bind (the bind manager skips invocation if connection already alive)
              QError *bind_err;
              QDN base_dn;
              ConnID conn_id;
              
              sc->IsSpecific( base_dn );
							conn_id = base_dn.GetConnID();
              if ( Manager()->BindManager()->BindAsync( conn_id, bind_err ) == FALSE )
                {
#ifdef _DEBUG
                  CString msg, err_str;
                  bind_err->Print( err_str );

                  msg.Format
                    ( "Bind failed: connection id '%d', error was:\n%s\n",
											conn_id, err_str );

									OutputDebugString( msg );
#endif
                  // Can't specify a search operation without a connection...
									delete bind_err;
                  errs++;
                  continue;
                }

              //
              // Create form search operation node specifier
							QFormSearchNode *node = new QFormSearchNode
								( Manager(),
									conn_id,
									form,
									ffi_target_left,
									ffi_left_list,
									base_dn,
									filter_str );

              srch_node_list.AddTail( node );
              node_count++;
            }
				}
		}

    //
    // Check that some initial search operations have been defined
    if ( node_count == 0 )
      {
#ifdef _DEBUG
        CString msg;
        msg.Format
          ( "Form search invocation failed : no initial searches possible for form '%s'.\n", form->Name() );

				OutputDebugString( msg );
#endif
        err = new QError( QERR_INSUFFICIENT_DATA );
        return FALSE;
      }

#ifdef _DEBUG
    //
    // XXX If some operations fail then this is only notified in the debugging version.
    // Add logging code when logging support is in place.
    if ( errs > 0 )
      {
        CString msg;
        msg.Format
          ( "Some search operations failed to initiate: '%d' ops failed.\n", errs );

				OutputDebugString( msg );
      }
#endif

    //
    // Invoke defined search operations 
    int attempted = 0;
    int failed = 0;

    InvokeOperations( attempted, failed );

		m_nOpsInvoked += attempted;
		m_nOpsFailed += failed;

		//
		// Send a report
		ReportOpCount();

		//
		// Return an error if no initial searches succeeded
    if ( failed == attempted )
      {
        err = new QError( QERR_SEARCHES_FAILED );
        return FALSE;
      }

    //
    // Return without error
		Manager()->RegisterRequest( this );
    return TRUE;
}

//////////////////////////////////////////////////////////////////////////////
// Invoke operations stored in the operation list (QFormSearch::op_list)
void QFormSearch::InvokeOperations( int &attempted, int &failed )
{
  WIN32_GUFF();

  POSITION pos = srch_node_list.GetHeadPosition();

  attempted = 0;
  failed = 0;

  while ( pos != NULL )
    {
      QFormSearchNode * node = srch_node_list.GetNext( pos );
      if ( node->Invoked() == FALSE )
        {
          QError *err;
					attempted++;
          if ( node->Invoke( err ) == FALSE ) 
            {
#ifdef _DEBUG
              CString msg, err_str;
              err->Print( msg );
              msg.Format
                ( "Search operation invocation failed: error given was '%s'.\n", err_str );

							OutputDebugString( msg );
#endif

							delete err;
              failed++;
            }
        }
    }
}

//////////////////////////////////////////////////////////////////////
// LDAP an incoming LDAP search result or error
BOOL QFormSearch::HandleLDAPMessage( LDAP *ld, LDAPMessage *lm, QError *& err )
{
  WIN32_GUFF();

  //
  // Get the operation record corresponding to this message
  QFormSearchNode *node;
  BOOL got_it = FALSE;

  POSITION this_node_pos, node_pos = srch_node_list.GetHeadPosition();
  while ( node_pos != NULL && got_it == FALSE)
    {
      this_node_pos = node_pos;
      node = srch_node_list.GetNext( node_pos );
      if ( node->IsTaskOwner( ld, ldap_msgid( lm ) ) )
        got_it = TRUE;
    }

  //
  // couldn't find the operation responsible for this message! Flag, though 
  // we'll attempt to continue
  if ( got_it == FALSE )
    {
      ldap_msgfree( lm );
      err = new QError( QERR_INVALID_ARGUMENT );
      return FALSE;
    }

  //
  // Dispatch the message to the operation handler
  BOOL ok = node->HandleLDAPMessage( ld, lm, err );
  if ( ok == FALSE )
    {
      //
      // The message was an or resulted in error
      if ( node->Done() == TRUE )
        {
					if ( node->Succeeded() == FALSE )
						{
							m_nOpsFailed = TRUE;
							ReportOpCount();
						}

          delete( node );
          srch_node_list.RemoveAt( this_node_pos );
        }
    }
  else
    {
      //
      // Get and process matches.
      PtrList<QEntry> match_list;
      UINT match_num = node->GetMatches( match_list );
      if ( match_num > 0 )
        {
					if ( node->TargetSearch() == TRUE )
						ProcessTargetMatches( node, match_list );
					else
						ProcessIntermediateMatches( node, match_list );
        }

			//
			// Get and process referrals. 
			QStringList ref_list;
			UINT ref_num = node->GetReferrals( ref_list );
			if ( ref_num > 0 )
				{
					ProcessReferrals( node, ref_list );
				}

			//
			// Remove the operation if finished
			if ( node->Done() )
				{
					delete( node );
					srch_node_list.RemoveAt( this_node_pos );
				}

			//
			// Report current operation counts
			ReportOpCount();
    }

  //
	// Check to see if request has completed
	if ( srch_node_list.GetCount() == 0 )
		// No outstanding searches
		{
      finished = TRUE;
			invoked = FALSE;

			QFormCompletionReport *pReport =
				new QFormCompletionReport( m_nOpsInvoked, m_nOpsFailed );

			Notify( DUAcomplete, static_cast<void *>( pReport ) );
		}

  return ok;
}

//////////////////////////////////////////////////////////////////////
// Process intermediate matches from a single search operation.
// Entries in 'match_list' are invalidated after this call.
void QFormSearch::ProcessIntermediateMatches
	( QFormSearchNode *node, PtrList<QEntry> &match_list )
{
  WIN32_GUFF();

	if ( match_list.GetCount() == 0 )
		return;

	//
	// Get remaining input fields
	QFormInput *ffi_target;
	PtrList<QFormInput> ffi_left_list;
	node->GetRemainingInput( &ffi_target, &ffi_left_list );

	if ( srch_opts.AutoChaseReferrals == TRUE )
		{
			POSITION pos = match_list.GetHeadPosition();
			while ( pos != NULL )
				{
					QEntry *entry = match_list.GetNext( pos );
					QSearchClass *sc = form->RuleSet()->FindMatchingClass( entry );
					if ( sc == NULL )
						//
						// Can't use this entry to continue the search as
						// no matching search rule found for it.
						{
							continue;
						}

					//
					// Find and use input fields that can be used against the given
					// search rule (QSearchClass). Try the target field input first,
					// then the context fields.
					TrySearch
						( entry->DN(), *sc, node->GetConnID(), ffi_target, ffi_target, ffi_left_list );

					ReportOpCount();

					POSITION ffi_pos = ffi_left_list.GetHeadPosition();
					while ( ffi_pos != NULL )
						{
							QFormInput *ffi = ffi_left_list.GetNext( ffi_pos );
							TrySearch
								( entry->DN(), *sc, node->GetConnID(), ffi, ffi_target, ffi_left_list );
						}
				}
		}

	//
	// Build and send off a report for the intermediate matches made
	QFormReport *report;
	if ( srch_opts.AutoChaseReferrals == TRUE )
		report = new QFormReport( match_list, FALSE, NULL );
	else
		report = new QFormReport( match_list, TRUE, node );

	Notify( DUAreport, static_cast<void *>( report ) );
}

//////////////////////////////////////////////////////////////////////
// Process intermediate matches from a single search operation.
// Entries in 'match_list' are invalidated after this call.
void QFormSearch::ProcessTargetMatches
	( QFormSearchNode *node, PtrList<QEntry> &match_list )
{
  WIN32_GUFF();

	if ( match_list.GetCount() == 0 )
		return;

	//
	// Build and send off a search results message
	QFormResult *result = new QFormResult( match_list );
	Notify( DUAresult, static_cast<void *>( result ) );
}

//////////////////////////////////////////////////////////////////////
// Process referrals returned from a single search operation.
void QFormSearch::ProcessReferrals
	( QFormSearchNode *node, QStringList &ref_list )
{
  WIN32_GUFF();

	POSITION pos = ref_list.GetHeadPosition();
	while ( pos != NULL )
		{
			CString ref_str = ref_list.GetNext( pos );

			LDAPURLDesc *ludp = NULL;
			char *uri_str = strdup( (LPCTSTR) ref_str );

			int url_ok = ldap_url_parse( uri_str, &ludp );
			if ( url_ok != LDAP_SUCCESS )
				{
					QLogFile::Log
						( "Failed to parse a returned referral:\n%s\n", ref_str );

					free( uri_str );
					continue;
				}

			//
			// Chase the referral
			QFormInput *ffi_target;
			PtrList<QFormInput> ffi_left_list;

			node->GetRemainingInput( &ffi_target, &ffi_left_list );
			TryReferral( ludp, ffi_target, ffi_left_list );

			//
			// Done with this referral
			if ( ludp != NULL )
				ldap_free_urldesc( ludp );

			free( uri_str );
		}
}

//////////////////////////////////////////////////////////////////////
// Chase the given referral
void QFormSearch::TryReferral
	( LDAPURLDesc *ludp, QFormInput *ffi_target, PtrList<QFormInput> &ffi_left_list )
{
	WIN32_GUFF();

	CString host, user, password;
	ConnID conn_id;

	//
	// Get host details from URL (set authentication to none)
	host = ludp->lud_host;
	user.Empty();
	password.Empty();

	conn_id = 
		Manager()->BindManager()->SpecifyConnection
			( host, ludp->lud_port, user, password );

	if ( conn_id == UNDEF_CONN_ID )
		return;

	//
	// Get base DN
	QDN base_dn;
	if ( base_dn.SetDN( Manager(), conn_id, CString( ludp->lud_dn ) ) == FALSE )
		{
			QLogFile::Log
				( "QFormSearch::TryReferral()\
					\nFailed to process referral, problem with DN:\
					\nDN: %s",
					ludp->lud_dn );

			//
			// Can't specify a search operation without a search base...
			return;
		}

  //
  // Initiate bind (the bind manager skips invocation if connection already alive)
  QError *err = NULL;
  if ( Manager()->BindManager()->RebindAsync( conn_id, err ) == FALSE )
    {
			// XXX User notify
		
			//
			// Can't specify a search operation without a connection...
			delete err;
			return;
    }

  //
  // Create form search operation node specifier
	QFormSearchNode *node = new QFormSearchNode
		( Manager(),
			conn_id,
			form,
			ffi_target,
			ffi_left_list,
			base_dn,
			CString( ludp->lud_filter ) );

	m_nOpsInvoked++;
	if ( node->Invoke( err ) == FALSE )
		{
			// XXX User notify
			m_nOpsFailed++;
			delete node;
			delete err;
		}
	else
		{
			//
			// If all went well add the search operation node
			// to the list
			srch_node_list.AddTail( node );
		}

	ReportOpCount();
}

//////////////////////////////////////////////////////////////////////
// Abort a search in progress
void QFormSearch::Abort()
{
  WIN32_GUFF();

  POSITION pos = srch_node_list.GetHeadPosition();
  while ( pos != NULL )
    {
      QFormSearchNode *node = srch_node_list.GetNext( pos );
      node->Abort();
      delete node;
    }

  srch_node_list.RemoveAll();
}

//////////////////////////////////////////////////////////////////////
// Is this request responsible for the given LDAP message?
BOOL QFormSearch::IsTaskOwner( LDAP *ld, int ldap_msg_id )
{
  WIN32_GUFF();

	POSITION pos = srch_node_list.GetHeadPosition();
	while ( pos != NULL )
		{
			QFormSearchNode *node = srch_node_list.GetNext( pos );
			if ( node->IsTaskOwner( ld, ldap_msg_id ) == TRUE )
				return TRUE;
		}

	return FALSE;
}

//////////////////////////////////////////////////////////////////////
// See if the given search rule can be used to generate a search
// against the given base entry and form field input (ffi_try).
// If yes then create a new search node (QFormSearchNode) and 
// invoke the search operation.
void QFormSearch::TrySearch
	( QDN &base_dn,
		QSearchClass &sc,
		ConnID conn_id,
		QFormInput *ffi_try,
		QFormInput *ffi_target_in, 
		PtrList<QFormInput> &ffi_left_list_in )
{
  WIN32_GUFF();

	//
	// Can the given base DN and search class be used to search
	// against the given form input?
	if ( sc.CanSearchFor( ffi_try->SearchClass() ) == FALSE )
		// Can't search using the given form field input
		{
			return;
		}

	//
	// Can search, so create a corresponding search specification
	// (a QFormSearchNode) and invoke it. Begin by computing the
	// remaining search input/criteria.
	QFormInput *ffi_target = ( ffi_try == ffi_target_in? NULL : ffi_target_in );

	PtrList<QFormInput> ffi_left_list;
	POSITION ffi_pos = ffi_left_list_in.GetHeadPosition();
	while ( ffi_pos != NULL )
		{
			QFormInput *ffi = ffi_left_list_in.GetNext( ffi_pos );
			if ( ffi != ffi_try )
				ffi_left_list.AddTail( ffi );
		}

  //
  // Generate search filter from search class and form field input string
  CString filter_str;
  if ( ffi_try->GenerateFilterString( filter_str ) == FALSE )
    {
#ifdef _DEBUG
      CString msg;
      msg.Format
        ( "Search filter generation failed : search class '%s'\n", sc.Name() );

			OutputDebugString( msg );
#endif

			return;
    }

  //
  // Initiate bind (the bind manager skips invocation if connection already alive)
  QError *err = NULL;
  if ( Manager()->BindManager()->RebindAsync( conn_id, err ) == FALSE )
    {
#ifdef _DEBUG
      CString msg, err_str;
      err->Print( err_str );
      msg.Format
        ( "Bind failed: error was:\n%s\n", err_str );

			OutputDebugString( msg );
#endif
      
			// Can't specify a search operation without a connection...
			delete err;
			return;
    }

  //
  // Create form search operation node specifier
	QFormSearchNode *node = new QFormSearchNode
		( Manager(), conn_id, form, ffi_target, ffi_left_list, base_dn, filter_str );

	m_nOpsInvoked++;
	if ( node->Invoke( err ) == FALSE )
		{
			m_nOpsFailed++;
			delete node;
		}
	else
		{
			//
			// All went OK...
			srch_node_list.AddTail( node );
		}

	ReportOpCount();
}

//////////////////////////////////////////////////////////////////////
// QFormSearch::NotifyCloseConnection
//
// Notify of abnormal server connection closure.
void QFormSearch::NotifyCloseConnection( ConnID conn_id )
{
	WIN32_GUFF();

	POSITION pos = srch_node_list.GetHeadPosition();
	while ( pos != NULL )
		{
			POSITION last_pos = pos;
			QFormSearchNode *pNode = srch_node_list.GetNext( pos );
			if ( pNode->Invoked() == TRUE && 
						pNode->Done() == FALSE &&
						pNode->GetConnID() == conn_id )
				{
					//
					// Abort this search operation as it's waiting
					// on the terminated connection
					pNode->Abort();
				}
		}

	//
	// If no searches left return complete report
	if ( srch_node_list.GetCount() == 0 )
		{
      finished = TRUE;
			invoked = FALSE;

			QFormCompletionReport *pReport = 
				new QFormCompletionReport( m_nOpsInvoked, m_nOpsFailed );

			Notify( DUAcomplete, static_cast<void *>( pReport ) );
		}
}

//////////////////////////////////////////////////////////////////////
// QFormReport
//////////////////////////////////////////////////////////////////////
QFormReport::QFormReport
	( PtrList<QEntry> &entry_list_arg,
		BOOL confirm_arg,
		QFormSearchNode *node,
		QFormReportType type_arg )
{
  WIN32_GUFF();

  type = type_arg;
	confirm = confirm_arg;
	originating_node = node;

  POSITION pos = entry_list_arg.GetHeadPosition();
  while ( pos != NULL )
    entry_list.AddTail( entry_list_arg.GetNext( pos ) );

	err = NULL;
}

QFormReport::QFormReport
	( CStringList &ref_list_arg,
		BOOL confirm_arg,
		QFormSearchNode *node,
		QFormReportType type_arg )
{
  WIN32_GUFF();

  type = type_arg;
	confirm = confirm_arg;
	originating_node = node;

  POSITION pos = ref_list_arg.GetHeadPosition();
  while ( pos != NULL )
    ref_list.AddTail( ref_list_arg.GetNext( pos ) );

	err = NULL;
}

QFormReport::QFormReport( QError *err_arg, QFormReportType type_arg )
{
	WIN32_GUFF();
  type = type_arg;
  err = err_arg;
	confirm = FALSE;
	originating_node = NULL;
}

QFormReport::QFormReport( int nInvoked, int nFailed, QFormReportType type_arg )
{
	WIN32_GUFF();
	m_nOpsInvoked = nInvoked;
	m_nOpsFailed = nFailed;
  type = type_arg;
	err = NULL;
	confirm = FALSE;
	originating_node = NULL;
}

QFormReport::~QFormReport()
{
	WIN32_GUFF();
  DeleteEntries();
  DeleteReferrals();
  DeleteError();
}

UINT QFormReport::GetEntries( PtrList<QEntry> &entry_list_get )
{
  WIN32_GUFF();

  POSITION pos = entry_list.GetHeadPosition();
  while ( pos != NULL )
    entry_list_get.AddTail( entry_list.GetNext( pos ) );

  entry_list.RemoveAll();
  return entry_list.GetCount();
}

UINT QFormReport::GetReferrals( CStringList &ref_list_get )
{
  WIN32_GUFF();

  POSITION pos = ref_list.GetHeadPosition();
  while ( pos != NULL )
    ref_list_get.AddTail( ref_list.GetNext( pos ) );

  ref_list.RemoveAll();
  return ref_list_get.GetCount();
}

BOOL QFormReport::GetOpCount( int &nInvoked, int &nFailed )
{
	if ( type == QROpCount )
		{
			nInvoked = m_nOpsInvoked;
			nFailed = m_nOpsFailed ;
			return TRUE;
		}

	return FALSE;
}

void QFormReport::GetError( QError *& get_err )
{
  WIN32_GUFF();

  get_err = err;
  err = NULL;
}

void QFormReport::DeleteEntries()
{
  WIN32_GUFF();

  POSITION pos = entry_list.GetHeadPosition();
  while ( pos != NULL )
    {
      QEntry *entry = entry_list.GetNext( pos );
      delete entry;
    }

  entry_list.RemoveAll();
}

void QFormReport::DeleteReferrals()
{
  WIN32_GUFF();
  ref_list.RemoveAll();
}

void QFormReport::DeleteError()
{
	WIN32_GUFF();
  if ( err != NULL )
    {
      delete err;
      err = NULL;
    }
}

QFormResult::QFormResult( PtrList<QEntry> &entry_list_arg )
{
  WIN32_GUFF();

  POSITION pos = entry_list_arg.GetHeadPosition();
  while ( pos != NULL )
    entry_list.AddTail( entry_list_arg.GetNext( pos ) );
}

QFormResult::~QFormResult()
{
	WIN32_GUFF();
  DeleteEntries();
}

UINT QFormResult::GetEntries( PtrList<QEntry> &entry_list_get )
{
  WIN32_GUFF();

  POSITION pos = entry_list.GetHeadPosition();
  while ( pos != NULL )
    entry_list_get.AddTail( entry_list.GetNext( pos ) );

  entry_list.RemoveAll();
  return entry_list_get.GetCount();
}

void QFormResult::DeleteEntries()
{
  WIN32_GUFF();

  POSITION pos = entry_list.GetHeadPosition();
  while ( pos != NULL )
    {
      QEntry *entry = entry_list.GetNext( pos );
      delete entry;
    }

  entry_list.RemoveAll();
}
