// QFormSearch.h: interface for the QFormSearch class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_QFORMSEARCH_H__45E54021_AF8C_11D2_89E9_CBE42BFD5600__INCLUDED_)
#define AFX_QFORMSEARCH_H__45E54021_AF8C_11D2_89E9_CBE42BFD5600__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "stdafx.h"
#include "QBindManager.h"
#include "QSearchClass.h"
#include "QFormField.h"
#include "QFormSearchNode.h"
#include "QRuleSet.h"
#include "QRequest.h"
#include "QFormOp.h"


// An intermediate report
typedef enum QFormReportType
{
  QRInterimMatches, 
  QRReferrals, 
  QRError, 
	QROpCount
};

// Report data structure
class AFX_EXT_CLASS QFormReport
{
friend class QFormSearch;

public:
  QFormReport
		( PtrList<QEntry> &match_list, BOOL confirm, QFormSearchNode *node, QFormReportType type = QRInterimMatches );

  QFormReport
		( CStringList &ref_list, BOOL confirm, QFormSearchNode *node, QFormReportType type = QRReferrals );

  QFormReport
		( QError *err, QFormReportType type = QRError );

	QFormReport
		( int nInvoked, int nFailed, QFormReportType type = QROpCount );

  ~QFormReport();

public:
  QFormReportType Type() { return type ; };
  UINT GetEntries( PtrList<QEntry> &entry_list );
  UINT GetReferrals( CStringList &ref_list );
  void GetError( QError *& err );
	BOOL GetOpCount( int &nInvoked, int &nFailed );

public:
	BOOL ContinuationRequest() { return confirm; } ;

private:
  void DeleteEntries();
  void DeleteReferrals();
  void DeleteError();

protected:
	QFormSearchNode *originating_node;

public:
  QFormReportType type;
  PtrList<QEntry> entry_list;
  CStringList ref_list;
  QError *err;
	BOOL confirm;
	int m_nOpsInvoked;
	int m_nOpsFailed;
};

// A final result. The requester may receive a number of these.
class AFX_EXT_CLASS QFormResult
{
public:
  QFormResult( PtrList<QEntry> &match_list );
  ~QFormResult();

public:
  UINT GetEntries( PtrList<QEntry> &match_list );

private:
  void DeleteEntries();

public:
  PtrList<QEntry> entry_list;
};

//
// Invocation options
class AFX_EXT_CLASS QFormSearchOpts
{
public:
	QFormSearchOpts()
		{
			AutoChaseReferrals = TRUE;
			AutoChaseLimit = -1;
			AutoSearch = TRUE;
			AutoSearchLimit = -1;
		};

public:
	BOOL AutoChaseReferrals;
	int AutoChaseLimit;
	BOOL AutoSearch;
	int AutoSearchLimit;
};

//
// The class implementing the form search request operation
class AFX_EXT_CLASS QFormSearch : public QRequest
{
public:
	QFormSearch
    ( QDSManager *ds, QRequestID req_id, QRequester *requester,
			QForm *form, QFormInput *ffi_target, PtrList<QFormInput> &ffi_list );

	virtual ~QFormSearch();

public:
	// Invoke the specified operation
  BOOL Invoke( QError *& err );

	// Check to see if this request is waiting on any requests
	// outstanding on the given connection identifier
	BOOL WaitingOn( ConnID conn_id );

	// Continue searching against the given entry
	BOOL Continue( QFormReport *report, PtrList<QEntry> &entry_list, QError *& err );

	// Continue searching against the given referral
	BOOL Continue( QFormReport *report, QStringList &ref_list, QError *& err );

	// Abort operation
  void Abort();

	// Notify request of abnormal connection termination
	void NotifyCloseConnection( ConnID conn_id );

public:
  // Handle an LDAP message 
  BOOL HandleLDAPMessage( LDAP *ld, LDAPMessage *lm, QError *& err );

  // Is this search responsible for the given message ID?
  BOOL IsTaskOwner( LDAP *ld, int ldap_msg_id );

private:
  void InvokeOperations( int &attempted, int &failed );
	void ProcessTargetMatches( QFormSearchNode *node, PtrList<QEntry> &match_list );
	void ProcessIntermediateMatches( QFormSearchNode *node, PtrList<QEntry> &match_list );
	void ProcessReferrals( QFormSearchNode *node, QStringList &ref_list );

	void TrySearch
		( QDN &dn,
			QSearchClass &sc,
			ConnID conn_id,
			QFormInput *ffi_try,
			QFormInput *ffi_target, 
			PtrList<QFormInput> &ffi_left_list );

	void TryReferral
		( LDAPURLDesc *ludp, QFormInput *ffi_target, PtrList<QFormInput> &ffi_left_list );

	void ReportOpCount()
		{
			QFormReport *report = new QFormReport( m_nOpsInvoked, m_nOpsFailed );
			Notify( DUAreport, static_cast<void *>( report ) );
		};

private:
  // Form search heuristics
  QForm *form;

  // User search input 
  QFormInput *ffi_target;
  PtrList<QFormInput> ffi_list;

  // List of outstanding LDAP search operations
  PtrList<QFormSearchNode> srch_node_list;

	// Search options
	QFormSearchOpts srch_opts;

	// Status
	int m_nOpsInvoked;
	int m_nOpsFailed;
};

#endif // !defined(AFX_QFORMSEARCH_H__45E54021_AF8C_11D2_89E9_CBE42BFD5600__INCLUDED_)
