// QReadEntry.h: interface for the QReadEntry class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_QREADENTRY_H__4402CB63_4825_11D3_89E9_C3CF89F5EB86__INCLUDED_)
#define AFX_QREADENTRY_H__4402CB63_4825_11D3_89E9_C3CF89F5EB86__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "QRequest.h"
#include "QEntry.h"
#include "ldap.h"

class AFX_EXT_CLASS QReadResult
{
public:
	QReadResult( QEntry *pEntry )
		{
			m_pEntry = pEntry;
		};

	~QReadResult()
		{
			if ( m_pEntry != NULL )
				delete m_pEntry;
		};

public:
	QEntry *GetEntry()
		{
			QEntry *pEntry = m_pEntry;
			m_pEntry = NULL;
			return pEntry;
		};

private:
	QEntry *m_pEntry;
};

typedef enum QReadReportType
{
	QRRInvoked
};

class AFX_EXT_CLASS QReadErrorReport
{
public:
	QReadErrorReport() { ; } ;

public:
	QError &Error() { return m_err; } ;
	QDN &DN() { return m_DN ; } ;
	CString &Host() { return m_host ; } ;

private:
	QError m_err;
	QDN m_DN;
	CString m_host;
};

class AFX_EXT_CLASS QReadCompletionReport
{
public:
	QReadCompletionReport() { ; } ;

public:
	QDN &DN() { return m_DN ; } ;
	CString &Host() { return m_host ; } ;

private:
	QDN m_DN;
	CString m_host;
};

class AFX_EXT_CLASS QReadReport
{
public:
	QReadReport( QReadReportType nType )	{	m_nType = nType ; } ;

public:
	QReadReportType Type() { return m_nType ; } ;
	QDN &DN() { return m_DN ; } ;
	CString &Host() { return m_host ; } ;

private:
	QReadReportType m_nType;
	QDN m_DN;
	CString m_host;
};

class AFX_EXT_CLASS QReadEntry : public QRequest  
{
public:
	QReadEntry
		( QDSManager *pManager, QRequestID nReqID, QRequester *pRequester, QDN &rDN );

	virtual ~QReadEntry();

public:
	// Invoke the read operation
	BOOL Invoke( QError *&rErr );

public:
	//
	// Virtual interfaces
  BOOL HandleLDAPMessage( LDAP *pLDAP, LDAPMessage *pLDAPMsg, QError *&rErr );

  // Check whether an LDAP operation was invoked by this request
  BOOL IsTaskOwner( LDAP *pLDAP, int nMsgID )
		{
			if  ( pLDAP == GetLDAPContext() && m_nMsgID == nMsgID )
				return TRUE;
			else
				return FALSE;
		};

	// Is this request waiting on operations on the given connection?
	BOOL WaitingOn( ConnID conn_id );

  // Abort the outstanding request
  void Abort();

	// Notify of abnormal server connection closure
	void NotifyCloseConnection( ConnID conn_id );

private:
	LDAP *GetLDAPContext()
		{
			return Manager()->BindManager()->GetLDAPContext( m_DN.GetConnID() );
		};

	BOOL ParseEntry( LDAP *pLDAP, LDAPMessage *pLDAPMsg, QError *&rErr );
	BOOL ParseReferral( LDAP *pLDAP, LDAPMessage *pLDAPMsg, QError *&rErr );
	BOOL ParseResult( LDAP *pLDAP, LDAPMessage *pLDAPMsg, QError *&rErr );

private:
	QDN m_DN;
	int m_nMsgID;
	QEntry *m_pResultEntry;
};

#endif // !defined(AFX_QREADENTRY_H__4402CB63_4825_11D3_89E9_C3CF89F5EB86__INCLUDED_)
