// QError.cpp: implementation of the QError class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Query32.h"
#include "QError.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

QError::QError( QErrCode e )
{
  errcode = e;
}

QError::QError( int ldaperr, LDAPMessage *lm )
{
  if ( ldaperr == LDAP_SUCCESS )
    errcode = QERR_OK;

  switch ( ldaperr )
    {
    case LDAP_OPERATIONS_ERROR:
      errcode = QERR_SERVICE;
      break; // LDAP_OPERATIONS_ERROR
      
    case LDAP_PROTOCOL_ERROR:
      errcode = QERR_NETWORK_ERROR;
      break; // LDAP_PROTOCOL_ERROR
      
    case LDAP_TIMELIMIT_EXCEEDED:
      errcode = QERR_TIME;
      break; // LDAP_TIMELIMIT_EXCEEDED
      
    case LDAP_SIZELIMIT_EXCEEDED:
      errcode = QERR_SIZE;
      break; // LDAP_SIZELIMIT_EXCEEDED

    case LDAP_COMPARE_FALSE:
      errcode = QERR_NOTHING_FOUND;
      break; // LDAP_COMPARE_FALSE

    case LDAP_STRONG_AUTH_NOT_SUPPORTED:
      errcode = QERR_SECURITY;
      break; // LDAP_STRONG_AUTH_NOT_SUPPORTED

    case LDAP_STRONG_AUTH_REQUIRED:
      errcode = QERR_SECURITY;
      break; // LDAP_STRONG_AUTH_REQUIRED

    case LDAP_NO_SUCH_ATTRIBUTE:
      errcode = QERR_ATTRIBUTE;
      break; // LDAP_NO_SUCH_ATTRIBUTE

    case LDAP_UNDEFINED_TYPE:
      errcode = QERR_ATTRIBUTE;
      break; // LDAP_UNDEFINED_TYPE

    case LDAP_INAPPROPRIATE_MATCHING:
      errcode = QERR_NOTHING_FOUND;
      break; // LDAP_INAPPROPRIATE_MATCHING

    case LDAP_CONSTRAINT_VIOLATION:
      errcode = QERR_SERVICE;
      break; // LDAP_CONSTRAINT_VIOLATION

    case LDAP_TYPE_OR_VALUE_EXISTS:
      errcode = QERR_UPDATE;
      break; // LDAP_TYPE_OR_VALUE_EXISTS

    case LDAP_INVALID_SYNTAX:
      errcode = QERR_ATTRIBUTE;
      break; // LDAP_INVALID_SYNTAX
 
    case LDAP_NO_SUCH_OBJECT:
      errcode = QERR_NAME;
      break; // LDAP_NO_SUCH_OBJECT

    case LDAP_ALIAS_PROBLEM:
      errcode = QERR_NAME;
      break; // LDAP_ALIAS_PROBLEM

    case LDAP_INVALID_DN_SYNTAX:
      errcode = QERR_NAME;
      break; // LDAP_INVALID_DN_SYNTAX

    case LDAP_IS_LEAF:
      errcode = QERR_SERVICE;
      break; // LDAP_IS_LEAF

    case LDAP_ALIAS_DEREF_PROBLEM:
      errcode = QERR_NAME;
      break; // LDAP_ALIAS_DEREF_PROBLEM

    case LDAP_INAPPROPRIATE_AUTH:
      errcode = QERR_SECURITY;
      break; // LDAP_INAPPROPRIATE_AUTH

    case LDAP_INVALID_CREDENTIALS:
      errcode = QERR_SECURITY;
      break; // LDAP_INVALID_CREDENTIALS

    case LDAP_INSUFFICIENT_ACCESS:
      errcode = QERR_SECURITY;
      break; // LDAP_INSUFFICIENT_ACCESS

    case LDAP_BUSY:
      errcode = QERR_SERVICE;
      break; // LDAP_BUSY

    case LDAP_UNAVAILABLE:
      errcode = QERR_SERVICE;
      break; // LDAP_UNAVAILABLE

    case LDAP_UNWILLING_TO_PERFORM:
      errcode = QERR_SERVICE;
      break; // LDAP_UNWILLING_TO_PERFORM

    case LDAP_LOOP_DETECT:
      errcode = QERR_SERVICE;
      break; // LDAP_LOOP_DETECT

    case LDAP_NAMING_VIOLATION:
      errcode = QERR_SERVICE;
      break; // LDAP_NAMING_VIOLATION

    case LDAP_OBJECT_CLASS_VIOLATION:
      errcode = QERR_ATTRIBUTE;
      break; // LDAP_OBJECT_CLASS_VIOLATION

    case LDAP_NOT_ALLOWED_ON_NONLEAF:
      errcode = QERR_SERVICE;
      break; // LDAP_NOT_ALLOWED_ON_NONLEAF

    case LDAP_NOT_ALLOWED_ON_RDN:
      errcode = QERR_ATTRIBUTE;
      break; // LDAP_NOT_ALLOWED_ON_RDN

    case LDAP_ALREADY_EXISTS:
      errcode = QERR_ATTRIBUTE;
      break; // LDAP_ALREADY_EXISTS

    case LDAP_NO_OBJECT_CLASS_MODS:
      errcode = QERR_UPDATE;
      break; // LDAP_NO_OBJECT_CLASS_MOD

    case LDAP_OTHER:
      errcode = QERR_INTERNAL;
      break; // LDAP_OTHER

    case LDAP_SERVER_DOWN:
      errcode = QERR_LOST_SERVER_CONNECTION;
      break; // LDAP_SERVER_DOWN

    case LDAP_LOCAL_ERROR:
      errcode = QERR_INTERNAL;
      break; // LDAP_LOCAL_ERROR

    case LDAP_ENCODING_ERROR:
      errcode = QERR_INTERNAL;
      break; // LDAP_ENCODING_ERROR

    case LDAP_DECODING_ERROR:
      errcode = QERR_INTERNAL;
      break; // LDAP_DECODING_ERROR

    case LDAP_AUTH_UNKNOWN:
      errcode = QERR_SECURITY;
      break; // LDAP_AUTH_UNKNOWN

    case LDAP_FILTER_ERROR:
      errcode = QERR_FILTER_ERROR;
      break; // LDAP_FILTER_ERROR

    default:
      errcode = QERR_SERVICE;
      break;
    }
}

QError::~QError()
{
}

void QError::Print( CString &str )
{
  WIN32_GUFF();

  str = "Unrecognized error!";
  if (errcode == QERR_OK)
    {
      str = "Operation successful";
      return;
    }

  switch ( errcode )
    {      
    case QERR_ATTRIBUTE:
      str = "Invalid attribute supplied.";
      break;
      
    case QERR_NAME:
      str = "Invalid directory entry name supplied.";
      break;
      
    case QERR_SERVICE:
      str = "Service error, cannot perform.";
      break;
      
    case QERR_ABANDONED:
      str = "Request abandoned.";
      break;
      
    case QERR_SECURITY:
      str = "Security problem, you have insufficient rights or invalid credentials.";
      break;
      
    case QERR_ABANDONFAILED:
      str = "Couldn't abandon request.";
      break;
      
    case QERR_UPDATE:
      str = "Couldn't update entry.";
      break;
      
    case QERR_DSAREFERRAL:
      str = "Couldn't refer request.";
      break;

    case QERR_NETWORK_ERROR:
      str = "Network problem, cannot perform.";
      break;
      
    case QERR_NOT_BOUND:
      str = "Not connected to directory.";
      break;
      
    case QERR_INTERNAL_LIMIT:
      str = "Internal limit reached.";
      break;
      
    case QERR_INTERNAL:
      str = "Internal error detected.";
      break;
      
    case QERR_INVALID_ARGUMENT:
      str = "Invalid information supplied.";
      break;
      
    case QERR_INSUFFICIENT_DATA:
      str = "Not enough search data supplied.";
      break;
      
    case QERR_NOTHING_FOUND:
      str = "Nothing found with given values.";
      break;
      
    case QERR_TIME:
      str = "Time limit reached.";
      break;
      
    case QERR_SIZE:
      str = "Data retrieval limit reached.";
      break;
      
    case QERR_ADMIN:
      str = "Administrative limit reached.";
      break;

    case QERR_SVC_BUSY:
      str = "The directory service is busy at present.";
      break;

    case QERR_SEARCHES_FAILED:
      str = "Some search operations failed.";
      break;

    case QERR_LOST_SERVER_CONNECTION:
      str = 
				"A directory server connection was lost.\
				 this may be due to a server or network error.";

      break;
    }
}

QError &QError::operator=( QError &rErr )
{
	errcode = rErr.ErrorCode();
	return *this;
}

