// QFormOp.h: interface for the QFormOp class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_QFORMOP_H__INCLUDED_)
#define AFX_QFORMOP_H__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "stdafx.h"
#include "QDSManager.h"
#include "QBindManager.h"
#include "QFormConfiguration.h"
#include "QDN.h"

//
// Class representing a single search operation
class QFormOp
{
public:
  QFormOp( QDSManager *ds, QForm *form, ConnID conn_id, QDN &dn, CString &filter );
  ~QFormOp();

public:
  // Invoke a search operation using the given search specification (return LDAP message ID)
  int Invoke( QError *&err );

  // Abort the operation in progress
  void Abort();

  // Get matches. Matches may be returned asynchronously.
  UINT GetMatches( PtrList<QEntry> &match_list );

  // Get returned referrals. Referrals may be returned asynchronously.
  UINT GetReferrals( QStringList &ref_list );

public:
  // Has this search been invoked
  BOOL Invoked() { return op_invoked; } ;
   
  // Finished yet?
  BOOL Done() { return op_done; } ;

	// Finished and succeeded?
	BOOL Succeeded() { return op_succeeded; } ;

  // Get associated LDAP context
  LDAP *GetLDAPContext() { return ds->BindManager()->GetLDAPContext( conn_id ) ; } ;

	// Get associated connection ID
	ConnID GetConnID() { return conn_id; } ;

	// Get associated DS context
	QDSManager *GetDS() { return ds; } ;

private:
	// Functions for handling different LDAP result messages.
	BOOL AddMatchingEntry( LDAPMessage *lm, QError *&err );
	BOOL AddReferrals( LDAPMessage *lm, QError *&err );
	BOOL ParseResult( LDAPMessage *lm, QError *&err );

public:
  // Handle an LDAP message 
  BOOL HandleLDAPMessage( LDAP *ld, LDAPMessage *lm, QError *& err );

  // Does this operation have the given message ID?
  BOOL IsTaskOwner( LDAP *ld, int ldap_msg_id_arg )
    { 
			if ( ld == GetLDAPContext() && ldap_msg_id_arg == ldap_msg_id )
				return TRUE;
			else
				return FALSE;
		}

	void Dump();

private:
  // State and search specification
  QDSManager *ds;
  QForm *form;
  ConnID conn_id;
  QDN base_dn;
  CString filter;
	int scope;
	int match_num;

private:
  // LDAP message ID of the invoked operation
  int ldap_msg_id;

  // Whether operation has been invoked
  BOOL op_invoked;

  // Whether operation has completed
  BOOL op_done;

	// Whether the operations completed successfully
	BOOL op_succeeded;

private:
  // Result storage
  CStringList referral_list;
  PtrList<QEntry> match_list;
};

template PtrList<QFormOp>;

#endif // AFX_QFORMOP_H__INCLUDED_